import React, { useState, useEffect, useRef } from "react";
import { useTheme } from "../hooks/useTheme";
import { tokenize } from "../utils/tokenizer";
import type { HighlighterProps, Token } from "../types";
import { 
  Copy, 
  Check, 
  ChevronDown, 
  ChevronUp, 
  Code
} from "lucide-react";

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = false,
  wrapLines = false,
  maxHeight = "400px",
  isDarkMode,
  expandable = false,
  className = "",
  style = {},
  customBackgroundColor,
  showCopyButton = true,
  fileName = "",
  showSearch = false,
  showShare = false,
  showThemeToggle = false,
  showDownload = false,
}) => {
  const { theme, isDarkMode: contextDarkMode } = useTheme();
  const [copied, setCopied] = useState(false);
  const [isExpanded, setIsExpanded] = useState(false);
  const codeRef = useRef<HTMLPreElement>(null);
  const currentDarkMode = isDarkMode ?? contextDarkMode;

  const copyToClipboard = async () => {
    try {
      await navigator.clipboard.writeText(code);
      setCopied(true);
      setTimeout(() => setCopied(false), 2000);
    } catch (err) {
      console.error("Failed to copy:", err);
    }
  };

  const getTokenClassName = (type: string): string => {
    const baseColors: Record<string, string> = {
      keyword: "text-pink-600 dark:text-pink-400",
      string: "text-green-600 dark:text-green-400",
      number: "text-blue-600 dark:text-blue-400",
      boolean: "text-blue-600 dark:text-blue-400",
      comment: "text-gray-500 dark:text-gray-400 italic",
      operator: "text-purple-600 dark:text-purple-400",
      punctuation: "text-gray-600 dark:text-gray-400",
      function: "text-yellow-600 dark:text-yellow-400",
      class: "text-red-600 dark:text-red-400",
      property: "text-indigo-600 dark:text-indigo-400",
      variable: "text-orange-600 dark:text-orange-400",
      default: "text-gray-900 dark:text-gray-100",
    };

    return baseColors[type] || baseColors.default;
  };

  const containerClasses = [
    "rounded-lg overflow-hidden",
    "border border-gray-200 dark:border-gray-800",
    className || ""
  ].join(" ");

  const headerClasses = "flex items-center justify-between px-4 py-2 bg-gray-50 dark:bg-gray-800 border-b border-gray-200 dark:border-gray-700";
  
  const codeContainerClasses = [
    "relative",
    currentDarkMode ? "bg-gray-900" : "bg-white",
    !isExpanded ? "overflow-hidden" : ""
  ].join(" ");

  const codeClasses = [
    "p-4 text-sm font-mono",
    "overflow-x-auto",
    wrapLines ? "whitespace-pre-wrap" : "whitespace-pre"
  ].join(" ");

  return (
    <div className={containerClasses}>
      <div className={headerClasses}>
        <div className="flex items-center space-x-2">
          <Code className="w-4 h-4 text-gray-500 dark:text-gray-400" />
          {fileName && (
            <span className="text-sm font-medium text-gray-700 dark:text-gray-300">
              {fileName}
            </span>
          )}
        </div>
        {showCopyButton && (
          <button
            onClick={copyToClipboard}
            className="p-1 hover:bg-gray-100 dark:hover:bg-gray-700 rounded"
          >
            {copied ? (
              <Check className="w-4 h-4 text-green-500" />
            ) : (
              <Copy className="w-4 h-4 text-gray-500 dark:text-gray-400" />
            )}
          </button>
        )}
      </div>

      <div className={codeContainerClasses} style={{ maxHeight: isExpanded ? "none" : maxHeight }}>
        <pre
          ref={codeRef}
          className={codeClasses}
          style={style}
        >
          <code>
            {code.split("\n").map((line, lineIndex) => (
              <div key={lineIndex} className="flex">
                {showLineNumbers && (
                  <span className="select-none mr-4 text-gray-400 dark:text-gray-600 text-right w-8">
                    {lineIndex + 1}
                  </span>
                )}
                <span className="flex-1">
                  {tokenize(line, language).map((token, tokenIndex) => (
                    <span
                      key={tokenIndex}
                      className={getTokenClassName(token.type)}
                    >
                      {token.content}
                    </span>
                  ))}
                </span>
              </div>
            ))}
          </code>
        </pre>

        {!isExpanded && (
          <div className="absolute bottom-0 left-0 right-0 h-24 bg-gradient-to-t from-white dark:from-gray-900 to-transparent pointer-events-none" />
        )}
      </div>

      {expandable && (
        <button
          onClick={() => setIsExpanded(!isExpanded)}
          className="w-full px-4 py-2 text-sm text-gray-700 dark:text-gray-300 bg-gray-50 dark:bg-gray-800 hover:bg-gray-100 dark:hover:bg-gray-700 flex items-center justify-center gap-2 transition-colors"
        >
          {isExpanded ? (
            <>
              <ChevronUp className="w-4 h-4" />
              Show Less
            </>
          ) : (
            <>
              <ChevronDown className="w-4 h-4" />
              Show More
            </>
          )}
        </button>
      )}
    </div>
  );
};

export default SyntaxHighlighter;
